/*
 * SPI.c
 *
 *  Created on: Jun 28, 2016
 *      Author: tbaruah
 */

#include "SPI.h"
#include "mode.h"

unsigned int Rfifo;

/**********************************************************************************************************************************************************
 Function: Init_SPI_Transfer
 Author:   Trinayan Baruah
 Purpose:  This function transmits the CFG word and also reads back the conversion results from the ADC.

 **********************************************************************************************************************************************************/
#ifdef RAC
unsigned int Init_SPI_Transfer(uint16_t Dataword)
{

	unsigned int Read_Data;
	while(!(*pREG_SPI0_STAT & BITM_SPI_STAT_RFE));		//	read RFIFO till it is empty
	Read_Data = *pREG_SPI0_RFIFO;
	Read_Data = 0;
	*pREG_SPI0_TWC	= 	1;								//	single byte instruction, no addr/data
	*pREG_SPI0_TFIFO = Dataword;						//	command ID
	while(!(*pREG_SPI0_STAT & BITM_SPI_STAT_TF));		//	wait till completion
	*pREG_SPI0_STAT = BITM_SPI_STAT_TF;					//	clear latch
	while(*pREG_SPI0_STAT & BITM_SPI_STAT_RFE); // Wait here as long as it is empty
	Read_Data =  (*pREG_SPI0_RFIFO ) & 0XFFFF;
	return Read_Data;
}
#endif
#ifndef RAC
unsigned int Init_SPI_Transfer(uint16_t Dataword)
{
	unsigned int Read_Data;
	*pREG_SPI0_TFIFO = Dataword;
	*pREG_SPI0_STAT = BITM_SPI_STAT_TF;					//	clear latch
	while(*pREG_SPI0_STAT &  BITM_SPI_STAT_RFE); // Wait here as long as it is  empty
	Read_Data =  ((*pREG_SPI0_RFIFO) & 0XFFFF );
	return Read_Data;
}
#endif
/**********************************************************************************************************************************************************
 Function: Init_SPI_AD7689
 Author:   Trinayan Baruah
 Purpose:  This function initializes the SPI registers

 **********************************************************************************************************************************************************/
void Init_SPI_AD7689(void)
{
	*pREG_SEC0_GCTL=ENUM_SEC_GCTL_EN;
	*pREG_SEC0_CCTL0=ENUM_SEC_CCTL_EN;


	*pREG_SPI0_CTL = 0x00;
	*pREG_SPI0_RXCTL = 0x00;
	*pREG_SPI0_TXCTL = 0x00;
#ifdef RAC
	*pREG_SPI0_CLK = 100 << BITP_SPI_CLK_BAUD; // Baud rate has to be changed
	*pREG_SPI0_TXCTL =  ENUM_SPI_TXCTL_TTI_EN | ENUM_SPI_TXCTL_TWC_EN | ENUM_SPI_TXCTL_ZERO | ENUM_SPI_TXCTL_TX_EN;
	*pREG_SPI0_RXCTL = 	ENUM_SPI_RXCTL_OVERWRITE  | ENUM_SPI_RXCTL_RX_EN;

#endif
#ifndef RAC
	*pREG_SPI0_CLK = 7 << BITP_SPI_CLK_BAUD;
	*pREG_SPI0_TXCTL =  ENUM_SPI_TXCTL_TTI_EN| ENUM_SPI_TXCTL_ZERO| ENUM_SPI_TXCTL_TX_EN;
	*pREG_SPI0_RXCTL = 	ENUM_SPI_RXCTL_OVERWRITE  | ENUM_SPI_RXCTL_RX_EN;
#endif
	*pREG_SPI0_CTL = ENUM_SPI_CTL_MASTER | ENUM_SPI_CTL_SCKMID
			| ENUM_SPI_CTL_SCKHI| ENUM_SPI_CTL_SIZE16 | ENUM_SPI_CTL_SW_SSEL
			| ENUM_SPI_CTL_MSB_FIRST;

	*pREG_SPI0_CTL |= BITM_SPI_CTL_EN; // Enable SPI0 since now it is safe to enable
	delay_afe(0XFFFF);
}
/**********************************************************************************************************************************************************
 Function:InitSpiPinmux
 Author:  Trinayan Baruah
 Purpose: This function configures the MUX and FER register for SPI communication

 **********************************************************************************************************************************************************/
void InitSpiPinmux(void) {
	*pREG_PORTC_MUX = SPI0_CLK_PORTC_MUX | SPI0_MISO_PORTC_MUX|SPI0_MOSI_PORTC_MUX;
	*pREG_PORTC_FER = SPI0_CLK_PORTC_FER | SPI0_MISO_PORTC_FER|SPI0_MOSI_PORTC_FER ;
}

/**********************************************************************************************************************************************************
 Function: delay_afe
 Author:   Trinayan Baruah
 Purpose:  Provides some delay after SPI configuration is done

 **********************************************************************************************************************************************************/

void delay_afe(unsigned int count) {
	for (volatile unsigned int i = 0; i < count; i++);
}
